# Generated by roxygen2: do not edit by hand

export(add_lake_context)
export(add_lake_depth)
export(add_weather_context)
export(assign_sites_to_lakes)
export(create_ray_geometries)
export(fetch_app)
export(fetch_app_upload)
export(fetch_calculate)
export(get_lake_boundary)
export(get_lake_depth)
export(lakefetch_options)
export(lakefetch_reset_options)
export(load_sites)
export(plot_fetch_bars)
export(plot_fetch_map)
export(plot_fetch_rose)
export(sanitize_filename)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_sf)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_sf)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(grDevices,rgb)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(osmdata,add_osm_feature)
importFrom(osmdata,opq)
importFrom(osmdata,osmdata_sf)
importFrom(sf,sf_use_s2)
importFrom(sf,st_area)
importFrom(sf,st_as_sf)
importFrom(sf,st_bbox)
importFrom(sf,st_boundary)
importFrom(sf,st_buffer)
importFrom(sf,st_cast)
importFrom(sf,st_centroid)
importFrom(sf,st_coordinates)
importFrom(sf,st_crs)
importFrom(sf,st_distance)
importFrom(sf,st_drop_geometry)
importFrom(sf,st_geometry)
importFrom(sf,st_geometry_type)
importFrom(sf,st_intersection)
importFrom(sf,st_intersects)
importFrom(sf,st_is_empty)
importFrom(sf,st_is_valid)
importFrom(sf,st_join)
importFrom(sf,st_linestring)
importFrom(sf,st_make_valid)
importFrom(sf,st_nearest_feature)
importFrom(sf,st_nearest_points)
importFrom(sf,st_point)
importFrom(sf,st_polygon)
importFrom(sf,st_read)
importFrom(sf,st_sf)
importFrom(sf,st_sfc)
importFrom(sf,st_transform)
importFrom(sf,st_union)
importFrom(stats,ave)
importFrom(stats,setNames)
importFrom(utils,read.csv)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
